using System;
using Gtk;
using OmniRig.Client;
using OmniRig.Interfaces;
using test;

public partial class MainWindow: Gtk.Window
{	
	private RigClient fOmniRig;    
    private CSSettings fServerConfig;

	public MainWindow(): base (Gtk.WindowType.Toplevel)
    {	
        Build();
		label1.ModifyFont(Pango.FontDescription.FromString("Courier Bold 24"));
		GLib.Timeout.Add(100, new GLib.TimeoutHandler(TimerEventHandler));
		
        try
        {
            bool err;
            fServerConfig = CSSettings.Load(out err, FileConfigPath());               
            fOmniRig = new RigClient();
            fOmniRig.Connected += new EventHandler(fOmniRig_Connected);			
            Connect(fServerConfig);			
        } 
        catch (Exception err)
        {
            MsgBox(err.Message, MessageType.Error, ButtonsType.Ok);                
        }
	}

    private ResponseType MsgBox(string text, MessageType icon, ButtonsType buttons)
    {
        Dialog dlg = new MessageDialog(this, DialogFlags.Modal, icon, buttons, text, new object[]{});
        int result = dlg.Run();
        dlg.Destroy();
        return (ResponseType)result;
    }
	
    private string FileConfigPath()
    {
        string path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
        return System.IO.Path.Combine(path, @"RS - Design/OmniRigTest/server.config");
    }

	protected void OnDeleteEvent (object sender, DeleteEventArgs a)
	{
        if (fOmniRig != null && !fServerConfig.RemoteConnection)
            fOmniRig.RemoveReference();
		Application.Quit ();
		a.RetVal = true;
	}
	
    private void fOmniRig_Connected(object sender, EventArgs e)
    {
        button2.Sensitive = button9.Sensitive = button10.Sensitive = 
			button11.Sensitive = button12.Sensitive = button13.Sensitive = 
				checkbutton1.Sensitive = true;
    }
	
	private bool TimerEventHandler()
    {
		if (fOmniRig == null || !fOmniRig.IsConnected)
			return true;
		
		Application.EventsPending();
		
        IRigData data = fOmniRig.GetRigData(RigNumber.RigA);
        if (data == null)
            return true;

        if (!data.IsOnline)
            label1.Text = label2.Text = String.Empty;
        else
        {
            label1.Text = data.Freq != 0 ? data.Freq.ToString() : data.FreqA.ToString();
            label2.Text = data.Mode.ToString();
        }

        label3.Text = data.StatusStr;
        Title = "Omni-Rig Test: " + data.RigType;
        checkbutton1.Active = data.Enabled;

        return true;
    }
		
    private void Connect(CSSettings config)
    {
        label3.Text = "Connect to server...";

        try
        {
            fOmniRig.Connect(config.RemoteConnection, config.Protocol, config.Host,
                    config.Port);
            fOmniRig.RigA.Enabled = checkbutton1.Active = true;
        } 
        catch (Exception err)
        {                
            MsgBox(err.Message, MessageType.Error, ButtonsType.Ok);
        }
    }  

    protected void RigSettings_Click(object sender, EventArgs e)
    {
        fOmniRig.ShowRigsSettingsDialog(this);
    }    

    protected void NetOption_Click(object sender, EventArgs e)
    {
        OptionDialog dlg = new OptionDialog();

        dlg.Remote = fServerConfig.RemoteConnection;
        dlg.Protocol = (int)fServerConfig.Protocol;
        dlg.Host = fServerConfig.Host;
        dlg.Port = fServerConfig.Port;

        try
        {
            if ((ResponseType)dlg.Run() == ResponseType.Ok)
            {
                CSSettings config = new CSSettings();
                config.RemoteConnection = dlg.Remote;
                config.Protocol = (NetProtocol)dlg.Protocol;
                config.Host = dlg.Host;
                config.Port = dlg.Port;

                if (CSSettings.Save(config, FileConfigPath()))
                {
                    fServerConfig = config;
                    fOmniRig.Connected -= new EventHandler(fOmniRig_Connected);
                    fOmniRig.Dispose();

                    label1.Text = label2.Text = label3.Text = String.Empty;
                    button2.Sensitive = false;
                                               
                    fOmniRig = new RigClient();
                    fOmniRig.Connected += new EventHandler(fOmniRig_Connected);
                    Connect(fServerConfig);
                }
            }
        } 
        finally
        {
            dlg.Destroy();
        }
    }

	protected void lsb_click (object sender, System.EventArgs e)
	{
		fOmniRig.RigA.Mode = RigParam.pmSSB_L;
	}

	protected void usb_click (object sender, System.EventArgs e)
	{
		fOmniRig.RigA.Mode = RigParam.pmSSB_U;
	}

	protected void am_click (object sender, System.EventArgs e)
	{
		fOmniRig.RigA.Mode = RigParam.pmAM;
	}

	protected void inc_click (object sender, System.EventArgs e)
	{
		fOmniRig.RigA.IncrementFreq(100, radiobutton1.Active ? RigVfo.AnyVfo : RigVfo.VfoA);
	}

	protected void dec_click (object sender, System.EventArgs e)
	{
		fOmniRig.RigA.DecrementFreq(100, radiobutton1.Active ? RigVfo.AnyVfo : RigVfo.VfoA);            
	}

	protected void enable_click (object sender, System.EventArgs e)
	{
		fOmniRig.RigA.Enabled = checkbutton1.Active;
	}
}
